﻿// (c) Copyright Microsoft Corporation.
// This source is subject to the Microsoft Public License (Ms-PL).
// Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
// All other rights reserved.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using SharpGIS.SyntaxHighlighting;
using System.Windows.Browser;

namespace SampleApp
{
	public partial class MainPage : UserControl
	{
		public MainPage()
		{
			InitializeComponent();

			Highlighter.SourceCode = LoadResource("CSharp.txt");
			HighlighterVb.SourceCode = LoadResource("VB.txt");
			HighlighterXaml.SourceCode = LoadResource("Xaml.txt");
			HighlighterJS.SourceCode = LoadResource("JS.txt");
			HighlighterCpp.SourceCode = LoadResource("Cpp.txt");
			HighlighterXml.SourceCode = LoadResource("Xml.txt"); 
		}

		private string LoadResource(string name)
		{
			using(System.IO.Stream s = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(
				string.Format("SampleApp.SampleCode.{0}", name))) {
				return new System.IO.StreamReader(s).ReadToEnd();
			}
		}


		private void Highlighter_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
		{
			e.Handled = true;
		}
		Canvas canvas;
		private void Highlighter_MouseRightButtonUp(object sender, MouseButtonEventArgs e)
		{
			if(string.IsNullOrEmpty((sender as SyntaxHighlightingTextBox).SelectedText))
				return;
			Point p = e.GetPosition(rightClickMenu.Parent as UIElement);
			rightClickMenu.HorizontalOffset = p.X;
			rightClickMenu.VerticalOffset = p.Y;
			rightClickMenu.IsOpen = true;
			if (canvas == null)
			{
				canvas = new Canvas() { Background = new SolidColorBrush(Color.FromArgb(1, 255, 255, 255)) };
				canvas.MouseLeftButtonDown += canvas_MouseLeftButtonDown;
			}
			if(canvas.Parent == null)
				LayoutRoot.Children.Add(canvas);
			rightClickMenu.DataContext = sender;
		}

		private void canvas_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			LayoutRoot.Children.Remove(canvas);
			rightClickMenu.IsOpen = false;
		}

		private void StackPanel_MouseEnter(object sender, MouseEventArgs e)
		{
			(sender as Panel).Background = LayoutRoot.Resources["hoverBack"] as Brush;
		}

		private void StackPanel_MouseLeave(object sender, MouseEventArgs e)
		{
			(sender as Panel).Background = new SolidColorBrush(Color.FromArgb(1, 255, 255, 255));
		}

		private void StackPanel_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			string tag = (sender as FrameworkElement).Tag as string;
			SyntaxHighlightingTextBox tb = (sender as FrameworkElement).DataContext as SyntaxHighlightingTextBox;
			switch (tag)
			{
				case "Cut":
					CopyText(tb.SelectedText);
					tb.SourceCode = tb.SourceCode.Substring(0, tb.SelectionStart) +
						tb.SourceCode.Substring(tb.SelectionStart + tb.SelectionLength);
					break;
				case "Copy":
					CopyText(tb.SelectedText);
					break;
				case "Paste":
					break;
			}
			canvas_MouseLeftButtonDown(null, null);
		}

		public static void CopyText(string text)
		{
			// document.window.clipboardData.setData(format, data);   
			var clipboardData = (ScriptObject)HtmlPage.Window.GetProperty("clipboardData");
			if (clipboardData != null)
			{
				bool success = (bool)clipboardData.Invoke("setData", "text", text);
				if (!success)
				{
					HtmlPage.Window.Alert("The text couldn't be copied into the clipboard.");
				}
			}
			else
			{
				HtmlPage.Window.Alert("The text will now attempt to be copied...");

				// Append a Flash embed element with the data encoded   
				string safeText = HttpUtility.UrlEncode(text);
				var elem = HtmlPage.Document.CreateElement("div");
				HtmlPage.Document.Body.AppendChild(elem);
				elem.SetProperty("innerHTML", "<embed src=\"" +
					"http://appengine.bravo9.com/copy-into-clipboard/clipboard.swf\" " +
					"FlashVars=\"clipboard=" + safeText + "\" width=\"0\" " +
					"height=\"0\" type=\"application/x-shockwave-flash\"></embed>");
			}
		}
	}
}